/* OpenVAS-Client
 *
 * Description: GUI elements for editing priorities
 *
 * Authors:
 * Joey Schulze <joey@infodrom.org>
 *
 * Copyright:
 * Copyright (C) 2009 Intevation GmbH
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2,
 * or, at your option, any later version as published by the Free
 * Software Foundation
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * In addition, as a special exception, you have
 * permission to link the code of this program with the OpenSSL
 * library (or with modified versions of OpenSSL that use the same
 * license as OpenSSL), and distribute linked combinations including
 * the two. You must obey the GNU General Public License in all
 * respects for all of the code used other than OpenSSL. If you
 * modify this file, you may extend this exception to your version
 * of the file, but you are not obligated to do so. If you do not
 * wish to do so, delete this exception statement from your version.
 */

/** @file
 * GUI elements for editing priorities.
 */

#include <includes.h>
#include "nessus_i18n.h"

#ifdef USE_GTK
#include <gtk/gtk.h>
#include "globals.h"
#include "context.h"
#include "priority.h"
#include "preferences.h"
#include "parser.h"
#include "parseutils.h"
#include "priority.h"
#include "nessus_plugin.h"
#include "plugin_infos.h"
#include "error_dlg.h"


enum {
  COL_OID,
  COL_PORT,
  COL_STATUS,
  NUM_COLS
};


/**
 * @brief Open a dialog to edit a priority chain.
 *
 * Consist of a notebook with one page for each port and beneath a table
 * with one set of radiobuttons for each message type found.
 * The new dialog is modal.  The supplied priority chain is altered
 * when the dialog is exited via the OK button.
 *
 * @param oid              OID of NVT.
 * @param priority[in,out] Priority chain
 *
 * @return The priority chain may be modified.
 */
void
edit_priority (char *oid, struct priority *priority)
{
  GtkWidget* dialog;
  GtkWidget* box;
  GtkWidget* frame;
  GtkWidget* vbox;
  GtkWidget* hbox;
  GtkWidget** radios;
  GtkWidget* button;
  GtkWidget* label;
  GtkWidget* notebook = NULL;
  struct context *context;
  struct nessus_plugin *plugin;
  gchar *str;
  char *port;
  int maxprios, nrprio, ismulti;

  context = context_by_type(Context, CONTEXT_REPORT);
  maxprios = priority_count(priority);
  ismulti = priority_is_multiport(priority);
  radios = emalloc(maxprios * sizeof(GtkWidget*));

  dialog = gtk_dialog_new_with_buttons(_("OpenVAS Priority Editor"),
				       NULL,
				       GTK_DIALOG_MODAL,
				       GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
				       GTK_STOCK_OK, GTK_RESPONSE_OK,
				       NULL);
   
  gtk_dialog_set_default_response(GTK_DIALOG(dialog), GTK_RESPONSE_CANCEL);
  gtk_window_set_resizable(GTK_WINDOW(dialog), FALSE);
  gtk_dialog_set_has_separator(GTK_DIALOG(dialog), FALSE);

  box = GTK_DIALOG(dialog)->vbox;
  gtk_box_set_spacing(GTK_BOX(box), 3);

  hbox = gtk_hbox_new(FALSE, FALSE);
  gtk_container_add(GTK_CONTAINER(box), hbox);
  str = g_strdup_printf (_("OID %s"), oid);
  label = gtk_label_new(str);
  g_free (str);
  gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);

  if (context && (plugin = nessus_plugin_get_by_oid(context->plugins, oid)) != NULL) {
    hbox = gtk_hbox_new(FALSE, FALSE);
    gtk_container_add(GTK_CONTAINER(box), hbox);
    label = gtk_label_new(plugin->name);
    gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
    // "Show Details"- Button
    button = gtk_button_new_with_label (_("Show Details"));
    g_signal_connect (GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(plugin_info_window_show_cb),
                      plugin->name);
    gtk_box_pack_start(GTK_BOX(box), button, FALSE, FALSE, 0);
  }

  if (ismulti) {
    notebook = gtk_notebook_new();
    gtk_container_add(GTK_CONTAINER(box), notebook);
  } else {
    hbox = gtk_hbox_new(FALSE, FALSE);
    gtk_container_add(GTK_CONTAINER(box), hbox);
    str = g_strdup_printf(_("Port: %s"), priority->port);
    label = gtk_label_new(str);
    g_free (str);
    gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_LEFT);
    gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
  }

  for (hbox=NULL,port=NULL,nrprio=0; nrprio < maxprios; nrprio++,priority = priority->next) {
    if (!port || strcmp(port, priority->port)) {
      hbox = gtk_hbox_new(FALSE, FALSE);
      if (ismulti) {
	label = gtk_label_new(priority->port);
	gtk_notebook_append_page(GTK_NOTEBOOK(notebook), hbox, label);
      } else
	gtk_container_add(GTK_CONTAINER(box), hbox);
      port = priority->port;
    }

    frame = gtk_frame_new(priority_type_to_str(priority->prio_old));
    gtk_container_add(GTK_CONTAINER(hbox), frame);

    vbox = gtk_vbox_new(FALSE, FALSE);
    gtk_container_add(GTK_CONTAINER(frame), vbox);

    button = gtk_radio_button_new_with_label_from_widget(NULL, priority_type_to_str(MSG_HOLE));
    gtk_object_set_data(GTK_OBJECT(button), "value", (void*)MSG_HOLE);
    gtk_object_set_data(GTK_OBJECT(button), "priority", priority);
    if (priority->prio_new == MSG_HOLE)
      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button), TRUE);
    gtk_container_add(GTK_CONTAINER(vbox), button);

    radios[nrprio] = button;

    button = gtk_radio_button_new_with_label_from_widget(GTK_RADIO_BUTTON(radios[nrprio]), priority_type_to_str(MSG_INFO));
    gtk_object_set_data(GTK_OBJECT(button), "value", (void*)MSG_INFO);
    gtk_object_set_data(GTK_OBJECT(button), "priority", priority);
    if (priority->prio_new == MSG_INFO)
      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button), TRUE);
    gtk_container_add(GTK_CONTAINER(vbox), button);

    button = gtk_radio_button_new_with_label_from_widget(GTK_RADIO_BUTTON(radios[nrprio]), priority_type_to_str(MSG_NOTE));
    gtk_object_set_data(GTK_OBJECT(button), "value", (void*)MSG_NOTE);
    gtk_object_set_data(GTK_OBJECT(button), "priority", priority);
    if (priority->prio_new == MSG_NOTE)
      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button), TRUE);
    gtk_container_add(GTK_CONTAINER(vbox), button);

    button = gtk_radio_button_new_with_label_from_widget(GTK_RADIO_BUTTON(radios[nrprio]), priority_type_to_str(MSG_FALSE));
    gtk_object_set_data(GTK_OBJECT(button), "value", (void*)MSG_FALSE);
    gtk_object_set_data(GTK_OBJECT(button), "priority", priority);
    if (priority->prio_new == MSG_FALSE)
      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button), TRUE);
    gtk_container_add(GTK_CONTAINER(vbox), button);
  }

  gtk_widget_show_all(dialog);
  if (gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_OK) {
    for (nrprio=0; nrprio < maxprios; nrprio++) {
      GSList *list = gtk_radio_button_group(GTK_RADIO_BUTTON(radios[nrprio]));
      while (list) {
 	GtkWidget *button = list->data;
	if (GTK_TOGGLE_BUTTON(button)->active) {
	  priority = gtk_object_get_data(GTK_OBJECT(button), "priority");
	  priority->prio_new = GPOINTER_TO_SIZE(gtk_object_get_data(GTK_OBJECT(button), "value"));
	}
	list = list->next;
      }
    }

  }
  gtk_widget_destroy(dialog);
}

/**
 * @brief Callback for the double-click in the editor window.
 *
 * A new modal dialog is opened when the current position in the
 * respective OID listing can be determined.  The TreeView is adjusted
 * with respect to the status of the particular line.
 *
 * @param treeview Respective TreeView
 * @param path     TreeView path
 * @param column   TreeView column
 * @param priorities Priority chain attached to the TreeView
 */
void
priorities_keypress_cb (GtkTreeView *treeview, GtkTreePath *path,
                        GtkTreeViewColumn *column, struct arglist *priorities)
{
  GtkTreeModel *model = gtk_tree_view_get_model(treeview);
  GtkTreeIter iter;
  char *oid;
  char *host;
  struct priority *priority;

  if (!gtk_tree_model_get_iter(model, &iter, path)) {
    fprintf(stderr, "priorities_keypress_cb: could not set iter from path\n");
    return;
  }

  host = gtk_object_get_data(GTK_OBJECT(treeview), "host");

  gtk_tree_model_get(model, &iter, COL_OID, &oid, -1);

  if ((priority = arg_get_value(priorities, oid)) == NULL) {
    fprintf(stderr, "priorities_keypress_cb: could not find script information for %s\n", oid);
    return;
  }

  edit_priority(oid, priority);
 
  if (priority_has_changed(priority)) {
    gtk_tree_store_set(GTK_TREE_STORE(model), &iter,
		       COL_STATUS, _("edited"),
		       -1);
    preference_store(host, oid, priority);
  } else {
    gtk_tree_store_set(GTK_TREE_STORE(model), &iter,
		       COL_STATUS, _("default"),
		       -1);
    preference_delete(host, oid);
  }
}

/**
 * @brief Display the main dialog for priorities.
 *
 * A new modal dialog is opened for all visible OpenVAS IDs in the
 * current report and overwritten in the current scope for all hosts
 * in the current report.  A double click opens a new modal dialog box
 * with all priorities and ports for the same OID and host.
 */
void
priorities_dialog (GtkWidget* w, struct arglist* ctrls)
{
  GtkWidget *dialog;
  GtkWidget *scrolledwindow;
  GtkWidget *treeview;
  GtkWidget* notebook = NULL;
  GtkTreeViewColumn *column;
  GtkCellRenderer *renderer;
  GtkTreeStore *model;
  GtkTreeIter current;
  struct arglist *priorities;
  struct arglist *priority;
  struct arglist *hostlist;
  struct arglist *v;
  char *status;
  int multi;

  dialog = gtk_dialog_new_with_buttons(_("OpenVAS Priorities Editor"),
				       NULL,
				       GTK_DIALOG_MODAL,
				       GTK_STOCK_OK, GTK_RESPONSE_OK,
				       NULL);
   
  priorities = priority = get_priorities(ctrls);
  
  if (priorities == NULL || priorities->next == NULL)
    {
      show_warning (_("Can not edit any priorities in empty report."));
      priorities_free (priorities);
      return;
    }

  multi = (priorities->next->next != NULL);

  if (multi) {
    notebook = gtk_notebook_new();
    gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox), notebook);
  }

  while (priority->next) {
    hostlist = priority->value;

    scrolledwindow = gtk_scrolled_window_new(NULL, NULL);

    if (multi) {
      GtkWidget *label = gtk_label_new(priority->name);
      gtk_notebook_append_page(GTK_NOTEBOOK(notebook), scrolledwindow, label);
    } else {
      gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox), scrolledwindow);
    }

    gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolledwindow),
				   GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

    treeview = gtk_tree_view_new();
    gtk_container_add(GTK_CONTAINER(scrolledwindow), treeview);
    gtk_tree_view_set_rules_hint(GTK_TREE_VIEW(treeview), TRUE);
    gtk_object_set_data(GTK_OBJECT(treeview), "host", priority->name);

    column = gtk_tree_view_column_new();
    gtk_tree_view_column_set_resizable(GTK_TREE_VIEW_COLUMN(column), TRUE);
    gtk_tree_view_column_set_title(column, _("OpenVAS Plugin ID"));
    gtk_tree_view_append_column(GTK_TREE_VIEW(treeview), column);
    renderer = gtk_cell_renderer_text_new();
    gtk_tree_view_column_pack_start(column, renderer, TRUE);
    gtk_tree_view_column_add_attribute(column, renderer, "text", COL_OID);

    column = gtk_tree_view_column_new();
    gtk_tree_view_column_set_resizable(GTK_TREE_VIEW_COLUMN(column), TRUE);
    gtk_tree_view_column_set_title(column, _("Port"));
    gtk_tree_view_append_column(GTK_TREE_VIEW(treeview), column);
    renderer = gtk_cell_renderer_text_new();
    gtk_tree_view_column_pack_start(column, renderer, TRUE);
    gtk_tree_view_column_add_attribute(column, renderer, "text", COL_PORT);

    column = gtk_tree_view_column_new();
    gtk_tree_view_column_set_resizable(GTK_TREE_VIEW_COLUMN(column), TRUE);
    gtk_tree_view_column_set_title(column, _("Status"));
    gtk_tree_view_append_column(GTK_TREE_VIEW(treeview), column);
    renderer = gtk_cell_renderer_text_new();
    gtk_tree_view_column_pack_start(column, renderer, TRUE);
    gtk_tree_view_column_add_attribute(column, renderer, "text", COL_STATUS);

    g_signal_connect(G_OBJECT(treeview), "row-activated",
		     G_CALLBACK(priorities_keypress_cb), hostlist);

    model = gtk_tree_store_new(NUM_COLS, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING);

    for (v = hostlist; v; v = v->next) {
      if (v->name) {

	if (priority_has_changed((struct priority *)v->value))
	  status = _("edited");
	else
	  status = _("default");

	gtk_tree_store_append(model, &current, NULL);
	gtk_tree_store_set(model, &current,
			   COL_OID, (char *)v->name,
			   COL_PORT, ((struct priority *)v->value)->port,
			   COL_STATUS, status,
			   -1);
      }
    }

    gtk_tree_view_set_model(GTK_TREE_VIEW(treeview), GTK_TREE_MODEL(model));
    g_object_unref(model);

    priority = priority->next;
  }

  gtk_window_set_default_size(GTK_WINDOW(dialog), 400, 220);

  /* execute and later remove the dialog */
  gtk_widget_show_all(dialog);
  gtk_dialog_run(GTK_DIALOG(dialog));
  gtk_widget_destroy(dialog);

  priorities_free(priorities);
}

#endif
